
#if !defined(FILES_H_INCLUDED)
#define FILES_H_INCLUDED

#include <stdio.h>
#include <string.h>
#include <time.h>

#include "except.h"

// #define BIGENDIAN
#undef BIGENDIAN

#ifdef BIGENDIAN
 #define BIG(something) (something)
 #define LITTLE(something) swap(something)
#else
 #define BIG(something) swap(something)
 #define LITTLE(something) (something)
#endif

unsigned char swap(const unsigned char &ref);
char swap(const char &ref);

unsigned short swap(const unsigned short &ref);
short swap(const short &ref);

unsigned int swap(const unsigned int &ref);
int swap(const int &ref);

long swap(const long &ref);
unsigned long swap(const unsigned long &ref);

double swap(const double &x);

/// ERROR CODES
const exception FILE_OPEN_ERROR    = 0x6B430001;
const exception FILE_READ_ERROR    = 0x6B430002;
const exception FILE_WRITE_ERROR   = 0x6B430003;
const exception FILE_SEEK_ERROR    = 0x6B430004;
const exception FILE_FORMAT_ERROR  = 0x6B430005;
const exception FILE_COMPILE_ERROR = 0x6B430006;

class CDiskFile {
public:
  CDiskFile();
  ~CDiskFile();
  void OpenRead(const char *name);
  void OpenRewrite(const char *name);
  void Close();
  void Read(void *buffer, unsigned long length);
  bool ReadLn(char *str, int max = 256);
  void Write(const void *buffer, unsigned long length);
  void PrintLn(const char* str);
  void Seek(long pos);
  long Size() const;
  static const char* ExceptionString(exception err);
protected:
  long filesize;
private:
  FILE *fp;
  void GetSize();
};

#ifndef FILES_NO_PDBFILE

class CPDBfile : private CDiskFile {
public:
  typedef struct {
    long offset;
    unsigned char attr;
    unsigned char uid[3];
    unsigned short size;
  } RecordInfo;
  typedef struct {
    long type;
    unsigned short id;
    long offset;
    unsigned short size;
  } ResourceInfo;

  CPDBfile();
  virtual ~CPDBfile();
  void OpenRead(const char *name);
  void OpenRewrite(const char *name, const char *dbname, long type, long creator, unsigned short entries, bool res);
  void Close();

  bool ResDB() const;
  unsigned short Entries() const;
  const char* Name() const;
  long CPDBfile::Creator() const;
  long CPDBfile::Type() const;

  void GetResourceInfo(unsigned short index, ResourceInfo &info) const;
  void GetRecordInfo(unsigned short index, RecordInfo &info) const;
  bool FindResource(long type, unsigned short id, unsigned short &index) const;
  unsigned short GetResourceSize(unsigned short index) const;
  void ReadResource(unsigned short index, void *buffer);
  unsigned short GetRecordSize(unsigned short index) const;
  void ReadRecord(unsigned short index, void *buffer);

  void WriteRecordInfo(unsigned short size);
  void WriteResourceInfo(unsigned short size,long type, unsigned short id);
  void WriteRecord(void *buffer);
  void WriteResource(void *buffer);

private:
#ifdef WIN32
#pragma pack(push,1)
#endif
  #define SIZEOF_PDBHEAD 78
  typedef struct {
    char name[32];
    unsigned short flags, version;
    long crdate,moddate,backupdate;
    long modnum;
    long app_info_offset,sort_info_offset;
    long type,creator;
    long unique_id_seed;
    long next_record_list_id;
    unsigned short entries;
  } PDBhead;
  #define SIZEOF_PDBRECORDINDEX 8
  typedef struct {
    long offset;
    unsigned char attr;
    unsigned char uid[3];
  } PDBRecordIndex;
  #define SIZEOF_PDBRESOURCEINDEX 10
  typedef struct {
    long type;
    unsigned short id;
    long offset;
  } PDBResourceIndex;
#ifdef WIN32
  #pragma pack(pop,1)
#endif
  PDBhead head;
  PDBResourceIndex *resources; PDBRecordIndex *records;
  unsigned short entries;
  long WriteOffset;
  unsigned short WriteIndexPrepare, WriteIndexFinal;
};

#endif


#endif
