
#include <stdio.h>

#include "except.h"
#include "files.h"

void rar2pdb(const char *src, const char *dst) {
  printf("%s -> %s\n",src,dst);
  CDiskFile rar;
  CPDBfile pdb;
  TRY {
    rar.OpenRead(src);
    const int blocksize = 32768;
    long total = rar.Size();
    int blocks = (total+blocksize-1)/blocksize;
    pdb.OpenRewrite(dst,src,'pRAR','pRAR',blocks,false);
    long offset;
    for (offset=0; offset<total; offset+=blocksize) {
      int now = (total-offset>blocksize) ? blocksize : total-offset;
      pdb.WriteRecordInfo(now);
    }
    for (offset=0; offset<total; offset+=blocksize) {
      int now = (total-offset>blocksize) ? blocksize : total-offset;
      char buffer[blocksize];
      rar.Read(buffer,now);
      pdb.WriteRecord(buffer);
    } 
    pdb.Close();
    rar.Close();
  }
  CATCH(err) {
    pdb.Close();
    rar.Close();
    THROW(err);
  }
}

ERRSTR_START
ERRSTR_CLASS(CDiskFile)
ERRSTR_END

const char *usage = "Usage: rar2pdb input.rar output.pdb\n";

int main(int argc, char *argv[],char *envp[]) {
  if (argc<3) {
    printf(usage);
    return 1;
  }
  bool ret = 0;
  TRY {
    rar2pdb(argv[1],argv[2]);
  }
  CATCH(err) {
    printf("Error: %s\n",ERRSTR(err));
    ret = 1;
  } ENDCATCH
  return ret;
}
